import React, { useState, useEffect, useRef } from 'react';
// Fallback wird fast nie benötigt, da wir inline Base64 und direkten Zugriff verwenden
import moselPanoramaImageFallback from '@/assets/mosel_panorama.jpg';
import { scrollToElement } from '@/lib/scrollUtils';
import { useQuery } from '@tanstack/react-query';
import type { Media } from '../../../shared/schema';
import OptimizedImage from '@/components/shared/OptimizedImage';
import { getMediaUrl } from '@/lib/media-utils';

// Optimierte URLs für LCP (80/20-Prinzip)
// Reduzierte Größe und optimale Qualität (800px statt 1280px)
// Hero-Bild-ID (Media-ID: 81) - Mosel-Panorama
const heroMediaId = 81;

// Optimierte URLs für verschiedene Formate, jetzt mit getMediaUrl erzeugt
const highQualityImageUrl = getMediaUrl(heroMediaId, { width: 800, format: 'avif', quality: 75 });

// Mini Inline Base64-kodiertes WebP, das sofort angezeigt wird
// Das ermöglicht eine sofortige Darstellung eines Platzhalters mit extrem geringer Größe
// Dies ist ein ultrakomprimiertes WebP-Bild des Panoramas, 20x11 Pixel, ~400 Bytes
const inlineImagePlaceholder = `data:image/webp;base64,UklGRoQAAABXRUJQVlA4WAoAAAAQAAAADwAABwAAQUxQSBcAAAABF0AmbQM2oP//ggIk//7dASQD8J+HKgBWUDggPAAAAJABAJ0BKhAAAAQADoaHYAJ0MEbA//ubR27dz91hzBXF/CxQAAD+8LhOlXl78QvVvGUGVvCQcX6oMqVs3fxSlLgNFoW7AvcFe3aAAAA=`;

// WebP-Fallback für Browser, die kein AVIF unterstützen
const webpImageUrl = getMediaUrl(heroMediaId, { width: 800, format: 'webp', quality: 75 });

const Hero: React.FC = () => {
  // Vereinfachte Zustandsverwaltung für LCP
  const [lcp, setLcp] = useState<number | null>(null);
  const [isHighQualityLoaded, setIsHighQualityLoaded] = useState(false);

  // Lade Media-Daten für zusätzliche Infos, verzögert nach initialem Rendering
  const { data: mediaList } = useQuery<Media[]>({
    queryKey: ['/api/media'],
    enabled: true,
    staleTime: 60000, // Cache für 1 Minute behalten
  });
  
  // LCP-Optimierungen und Metriken zur Überwachung
  useEffect(() => {
    if (typeof window === 'undefined') return;
    
    // Prüfen, ob wir auf einem mobilen Gerät sind (Viewport-Breite unter 768px)
    const isMobile = window.innerWidth < 768;
    
    // HTML-Präload-Fetch für das Hero-Bild (niedrigere Priorität auf mobilen Geräten)
    const link = document.createElement('link');
    link.rel = 'preload';
    link.href = highQualityImageUrl;
    link.as = 'image';
    link.type = 'image/avif';
    // Verwende fetchpriority für moderne Browser - weniger wichtig auf mobilen Geräten
    link.setAttribute('fetchpriority', isMobile ? 'low' : 'high');
    document.head.appendChild(link);

    // WebP-Fallback auch vorladen
    const linkWebp = document.createElement('link');
    linkWebp.rel = 'preload';
    linkWebp.href = webpImageUrl;
    linkWebp.as = 'image';
    linkWebp.type = 'image/webp';
    document.head.appendChild(linkWebp);
    
    // Performance-Timing für LCP überwachen mit PerformanceObserver
    const lcpObserver = new PerformanceObserver((entryList) => {
      const entries = entryList.getEntries();
      const lastEntry = entries[entries.length - 1];
      setLcp(lastEntry.startTime);
      
      // Debug-Info für Optimierung
      console.log('LCP erfasst:', Math.round(lastEntry.startTime), 'ms');
    });
    
    // Beobachter für LCP-Metriken registrieren
    lcpObserver.observe({ type: 'largest-contentful-paint', buffered: true });

    // Überprüfen, ob das Bild bereits im Browser-Cache ist
    // Einfacher Check ohne globale Variablen
    const img = new Image();
    img.onload = () => setIsHighQualityLoaded(true);
    // Niedrigere Priorität für Bildladen auf Mobilgeräten
    if (isMobile) {
      img.loading = 'lazy';
    } else {
      img.setAttribute('fetchpriority', 'high');
    }
    img.src = highQualityImageUrl;
    
    // Hinweis: Auf Mobilgeräten ist der Text wichtiger als das Hintergrundbild
    if (isMobile) {
      // Füge Inline-Styles hinzu, die das LCP-Element priorisieren
      const lcpElement = document.getElementById('hero-description');
      if (lcpElement) {
        lcpElement.style.visibility = 'visible';
        lcpElement.style.opacity = '1';
      }
    }
    
    return () => {
      lcpObserver.disconnect();
      // Links aus dem DOM entfernen, falls sie noch existieren
      try {
        document.head.removeChild(link);
        document.head.removeChild(linkWebp);
      } catch (e) {
        // Ignorieren, falls Links bereits entfernt wurden
      }
    };
  }, []);
  
  // Definierte benutzerdefinierte Farben aus der Tailwind-Konfiguration
  const colors = {
    forest: {
      DEFAULT: '#2B392A',
      light: '#495F41',
    },
    rose: {
      DEFAULT: '#D8848E',
      light: '#E5B0B6',
    },
    fog: {
      DEFAULT: '#F7F7F2',
      dark: '#E8E8E0',
    },
    beige: {
      DEFAULT: '#F2EAE4',
      dark: '#E5D6CC',
    }
  };

  return (
    <section className="relative h-[90vh] min-h-[600px] overflow-hidden" style={{ backgroundColor: colors.forest.DEFAULT }}>
      <div className="absolute inset-0">
        {/* Progressive Image Loading Strategy: 
            1. Sofortige Anzeige eines ultra-komprimierten Base64-Inlines als LQIP
            2. Hochauflösendes AVIF-Bild wird über `srcset` mit `picture` für Browser-Kompatibilität geladen
            3. WebP als Fallback, wenn AVIF nicht unterstützt wird
            4. Alle übergebenen fetchPriority, loading und decoding Attribute helfen dem Browser zu verstehen, dass dieses Bild wichtig ist 
         */}
        <div 
          style={{ 
            backgroundImage: `url(${inlineImagePlaceholder})`,
            backgroundSize: 'cover',
            backgroundPosition: 'center',
            position: 'absolute',
            top: 0,
            left: 0,
            right: 0,
            bottom: 0,
            zIndex: 0
          }}
          aria-hidden="true"
        ></div>

        <picture>
          <source 
            srcSet={highQualityImageUrl} 
            type="image/avif" 
          />
          <source 
            srcSet={webpImageUrl} 
            type="image/webp" 
          />
          <img 
            src={moselPanoramaImageFallback}
            alt="Panoramablick auf die berühmte Moselschleife mit Weinbergen und Dörfern"
            className="w-full h-full"
            loading="eager"
            decoding="sync"
            width="1920"
            height="1080"
            style={{ 
              objectFit: 'cover',
              position: 'relative', 
              zIndex: 1
            }}
          />
        </picture>

        {/* Overlay für besseres Text-Kontrast */}
        <div className="absolute inset-0 z-[2]" 
          style={{ 
            background: `linear-gradient(to bottom, ${colors.forest.DEFAULT}4D, ${colors.forest.DEFAULT}B3)` 
          }}
        ></div>
      </div>
      
      <div className="relative flex items-center justify-center h-full container mx-auto px-4 sm:px-6 lg:px-8 z-[2]">
        <div className="text-center max-w-3xl mx-auto">
          <h1 className="text-4xl sm:text-5xl md:text-6xl font-playfair font-bold text-white leading-tight mb-6">
            Erleben Sie Mosel-Magie am <span className="italic" style={{ color: colors.rose.light }}>Calmont</span>
          </h1>
          <p 
            className="text-lg md:text-xl mb-8 max-w-2xl mx-auto" 
            style={{ color: colors.fog.DEFAULT }}
            id="hero-description"
            data-lcp="true"
          >
            <span 
              className="lcp-text" 
              style={{ display: 'inline-block' }}
            >
              Einzigartige Ferienwohnungen in Bremm direkt am steilsten Weinberg Europas – der perfekte Ort für Weinliebhaber und Aktivurlauber.
            </span>
          </p>
          <div className="flex flex-col sm:flex-row justify-center gap-4">
            <a 
              href="#booking" 
              className="inline-flex items-center justify-center px-8 py-3 border border-transparent text-base font-medium rounded-md shadow-sm text-white transition-colors focus:outline-none focus:ring-2 focus:ring-offset-2 hover:opacity-90"
              style={{ 
                backgroundColor: colors.rose.DEFAULT
              }}
              onClick={(e) => {
                e.preventDefault();
                scrollToElement('booking');
              }}
            >
              Verfügbarkeit prüfen
            </a>
            <a 
              href="#apartments" 
              className="inline-flex items-center justify-center px-8 py-3 border border-transparent text-base font-medium rounded-md text-white transition-colors focus:outline-none focus:ring-2 focus:ring-offset-2 hover:opacity-90"
              style={{ 
                backgroundColor: colors.forest.light
              }}
              onClick={(e) => {
                e.preventDefault();
                scrollToElement('apartments');
              }}
            >
              Wohnungen entdecken
            </a>
          </div>
        </div>
      </div>
      
      <div className="absolute bottom-0 left-0 right-0">
        <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 1440 100" className="w-full h-auto" style={{ color: colors.fog.DEFAULT }}>
          <path fill="currentColor" fillOpacity="1" d="M0,64L80,58.7C160,53,320,43,480,48C640,53,800,75,960,75C1120,75,1280,53,1360,42.7L1440,32L1440,100L1360,100C1280,100,1120,100,960,100C800,100,640,100,480,100C320,100,160,100,80,100L0,100Z"></path>
        </svg>
      </div>
    </section>
  );
};

export default Hero;
